@@ -181,6 +181,12 @@ static void news_folder_destroy(Folder *folder)
 		remove_dir_recursive(dir);
 	g_free(dir);
 
+	dir = g_strconcat(get_news_cache_dir(), G_DIR_SEPARATOR_S,
+			  folder->account->nntp_server, NULL);
+	if (is_dir_exist(dir))
+		g_rmdir(dir);
+	g_free(dir);
+
 	folder_remote_folder_destroy(REMOTE_FOLDER(folder));
 }
 
