@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2008 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -346,16 +346,16 @@ static PrefParam param[] = {
 	/* Junk mail */
 	{"enable_junk", "FALSE", &prefs_common.enable_junk, P_BOOL},
 #ifdef G_OS_WIN32
-	{"junk_learn_command", "bsfilterw -su", &prefs_common.junk_learncmd,
-	 P_STRING},
-	{"nojunk_learn_command", "bsfilterw -cu",
+	{"junk_learn_command", "bsfilterw -C -s -u",
+	 &prefs_common.junk_learncmd, P_STRING},
+	{"nojunk_learn_command", "bsfilterw -c -S -u",
 	 &prefs_common.nojunk_learncmd, P_STRING},
 	{"junk_classify_command", "bsfilterw",
 	 &prefs_common.junk_classify_cmd, P_STRING},
 #else
-	{"junk_learn_command", "bogofilter -s -I", &prefs_common.junk_learncmd,
-	 P_STRING},
-	{"nojunk_learn_command", "bogofilter -n -I",
+	{"junk_learn_command", "bogofilter -N -s -I",
+	 &prefs_common.junk_learncmd, P_STRING},
+	{"nojunk_learn_command", "bogofilter -n -S -I",
 	 &prefs_common.nojunk_learncmd, P_STRING},
 	{"junk_classify_command", "bogofilter -I",
 	 &prefs_common.junk_classify_cmd, P_STRING},
@@ -462,6 +462,11 @@ static PrefParam param[] = {
 	{"ext_sendmail_cmd", DEFAULT_SENDMAIL_CMD, &prefs_common.extsend_cmd,
 	 P_STRING},
 
+	/* Update check */
+	{"auto_update_check", "TRUE", &prefs_common.auto_update_check, P_BOOL},
+	{"use_http_proxy", "FALSE", &prefs_common.use_http_proxy, P_BOOL},
+	{"http_proxy_host", NULL, &prefs_common.http_proxy_host, P_STRING},
+
 	/* Advanced */
 	{"strict_cache_check", "FALSE", &prefs_common.strict_cache_check,
 	 P_BOOL},
