@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -383,9 +383,9 @@ static gint set_nonblocking_mode(gint fd, gboolean nonblock)
 	sock = sock_find_from_fd(fd);
 	if (sock) {
 		if (nonblock) {
-			SOCK_SET_FLAGS(sock->flags, SOCK_NONBLOCK);
+			SOCK_SET_FLAGS(sock->flags, SYL_SOCK_NONBLOCK);
 		} else {
-			SOCK_UNSET_FLAGS(sock->flags, SOCK_NONBLOCK);
+			SOCK_UNSET_FLAGS(sock->flags, SYL_SOCK_NONBLOCK);
 		}
 	}
 	debug_print("set nonblocking mode to %d\n", nonblock);
@@ -418,9 +418,9 @@ gint sock_set_nonblocking_mode(SockInfo *sock, gboolean nonblock)
 	ret = set_nonblocking_mode(sock->sock, nonblock);
 	if (ret == 0) {
 		if (nonblock) {
-			SOCK_SET_FLAGS(sock->flags, SOCK_NONBLOCK);
+			SOCK_SET_FLAGS(sock->flags, SYL_SOCK_NONBLOCK);
 		} else {
-			SOCK_UNSET_FLAGS(sock->flags, SOCK_NONBLOCK);
+			SOCK_UNSET_FLAGS(sock->flags, SYL_SOCK_NONBLOCK);
 		}
 	}
 
@@ -882,7 +882,7 @@ SockInfo *sock_connect(const gchar *hostname, gushort port)
 	sockinfo->hostname = g_strdup(hostname);
 	sockinfo->port = port;
 	sockinfo->state = CONN_ESTABLISHED;
-	sockinfo->flags = SOCK_CHECK_IO;
+	sockinfo->flags = SYL_SOCK_CHECK_IO;
 
 	sock_list = g_list_prepend(sock_list, sockinfo);
 
@@ -922,7 +922,7 @@ static gboolean sock_connect_async_cb(GIOChannel *source,
 	conn_data->channel = NULL;
 	g_io_channel_unref(source);
 
-	if (condition & (G_IO_ERR | G_IO_HUP)) {
+	if (condition & (G_IO_ERR | G_IO_HUP | G_IO_NVAL)) {
 		debug_print("sock_connect_async_cb: condition = %d\n",
 			    condition);
 		fd_close(fd);
@@ -939,6 +939,7 @@ static gboolean sock_connect_async_cb(GIOChannel *source,
 	}
 
 	if (val != 0) {
+		debug_print("getsockopt(SOL_SOCKET, SO_ERROR) returned error\n");
 		fd_close(fd);
 		sock_connect_address_list_async(conn_data);
 		return FALSE;
@@ -950,7 +951,7 @@ static gboolean sock_connect_async_cb(GIOChannel *source,
 	sockinfo->hostname = g_strdup(conn_data->hostname);
 	sockinfo->port = conn_data->port;
 	sockinfo->state = CONN_ESTABLISHED;
-	sockinfo->flags = SOCK_NONBLOCK;
+	sockinfo->flags = SYL_SOCK_NONBLOCK;
 
 	sock_list = g_list_prepend(sock_list, sockinfo);
 
@@ -1086,9 +1087,9 @@ static gint sock_connect_address_list_async(SockConnectData *conn_data)
 	conn_data->cur_addr = conn_data->cur_addr->next;
 
 	conn_data->channel = g_io_channel_unix_new(sock);
-	conn_data->io_tag = g_io_add_watch(conn_data->channel,
-					   G_IO_OUT | G_IO_ERR | G_IO_HUP,
-					   sock_connect_async_cb, conn_data);
+	conn_data->io_tag = g_io_add_watch
+		(conn_data->channel, G_IO_OUT | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
+		 sock_connect_async_cb, conn_data);
 
 	return 0;
 }
