@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2005 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2009 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,11 +43,6 @@
 ProgressDialog *progress_dialog_create(void)
 {
 	ProgressDialog *progress;
-	GtkWidget *dialog;
-	GtkWidget *hbox;
-	GtkWidget *label;
-	GtkWidget *cancel_btn;
-	GtkWidget *progressbar;
 	GtkWidget *scrolledwin;
 	GtkWidget *treeview;
 	GtkListStore *store;
@@ -55,7 +50,81 @@ ProgressDialog *progress_dialog_create(void)
 	GtkTreeViewColumn *column;
 	GtkCellRenderer *renderer;
 
-	debug_print(_("Creating progress dialog...\n"));
+	progress = progress_dialog_simple_create();
+
+	scrolledwin = gtk_scrolled_window_new(NULL, NULL);
+	gtk_widget_show(scrolledwin);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(progress->window)->vbox),
+			   scrolledwin, TRUE, TRUE, 0);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
+				       GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwin),
+					    GTK_SHADOW_IN);
+
+	store = gtk_list_store_new(PROG_N_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING,
+				   G_TYPE_STRING, G_TYPE_STRING,
+				   G_TYPE_POINTER);
+
+	treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+	g_object_unref(G_OBJECT(store));
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), TRUE);
+	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
+	gtk_widget_show(treeview);
+	gtk_container_add(GTK_CONTAINER(scrolledwin), treeview);
+	gtk_widget_set_size_request(treeview, -1, 120);
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
+	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
+
+	renderer = gtk_cell_renderer_pixbuf_new();
+	g_object_set(renderer, "xalign", 0.5, NULL);
+	column = gtk_tree_view_column_new_with_attributes
+		(NULL, renderer, "pixbuf", PROG_COL_PIXBUF, NULL);
+	gtk_tree_view_column_set_alignment(column, 0.5);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_set_fixed_width(column, 20);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
+
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Account"), renderer, "text", PROG_COL_NAME, NULL);
+	gtk_tree_view_column_set_resizable(column, TRUE);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_set_fixed_width(column, 120);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
+
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Status"), renderer, "text", PROG_COL_STATUS, NULL);
+	gtk_tree_view_column_set_resizable(column, TRUE);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_set_fixed_width(column, 80);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
+
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes
+		(_("Progress"), renderer, "text", PROG_COL_PROGRESS, NULL);
+	gtk_tree_view_column_set_resizable(column, TRUE);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
+
+	progress->treeview = treeview;
+	progress->store    = store;
+
+	return progress;
+}
+
+ProgressDialog *progress_dialog_simple_create(void)
+{
+	ProgressDialog *progress;
+	GtkWidget *dialog;
+	GtkWidget *hbox;
+	GtkWidget *label;
+	GtkWidget *cancel_btn;
+	GtkWidget *progressbar;
+
+	debug_print("Creating progress dialog\n");
 	progress = g_new0(ProgressDialog, 1);
 
 	dialog = gtk_dialog_new();
@@ -90,60 +159,12 @@ ProgressDialog *progress_dialog_create(void)
 			   FALSE, FALSE, 0);
 	gtk_widget_show(progressbar);
 
-	scrolledwin = gtk_scrolled_window_new(NULL, NULL);
-	gtk_widget_show(scrolledwin);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), scrolledwin,
-			   TRUE, TRUE, 0);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwin),
-				       GTK_POLICY_AUTOMATIC,
-				       GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwin),
-					    GTK_SHADOW_IN);
-
-	store = gtk_list_store_new(PROG_N_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING,
-				   G_TYPE_STRING, G_TYPE_POINTER);
-
-	treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-	g_object_unref(G_OBJECT(store));
-	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), TRUE);
-	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
-	gtk_widget_show(treeview);
-	gtk_container_add(GTK_CONTAINER(scrolledwin), treeview);
-	gtk_widget_set_size_request(treeview, -1, 120);
-
-	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
-	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
-
-	renderer = gtk_cell_renderer_pixbuf_new();
-	g_object_set(renderer, "xalign", 0.5, NULL);
-	column = gtk_tree_view_column_new_with_attributes
-		(NULL, renderer, "pixbuf", PROG_COL_PIXBUF, NULL);
-	gtk_tree_view_column_set_alignment(column, 0.5);
-	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
-	gtk_tree_view_column_set_fixed_width(column, 20);
-	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
-
-	renderer = gtk_cell_renderer_text_new();
-	column = gtk_tree_view_column_new_with_attributes
-		(_("Account"), renderer, "text", PROG_COL_NAME, NULL);
-	gtk_tree_view_column_set_resizable(column, TRUE);
-	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
-	gtk_tree_view_column_set_fixed_width(column, 160);
-	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
-
-	renderer = gtk_cell_renderer_text_new();
-	column = gtk_tree_view_column_new_with_attributes
-		(_("Status"), renderer, "text", PROG_COL_STATUS, NULL);
-	gtk_tree_view_column_set_resizable(column, TRUE);
-	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_FIXED);
-	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
-
 	progress->window      = dialog;
 	progress->label       = label;
 	progress->cancel_btn  = cancel_btn;
 	progress->progressbar = progressbar;
-	progress->treeview    = treeview;
-	progress->store       = store;
+	progress->treeview    = NULL;
+	progress->store       = NULL;
 
 	return progress;
 }
@@ -175,7 +196,7 @@ void progress_dialog_set_percentage(ProgressDialog *progress,
 
 void progress_dialog_append(ProgressDialog *progress, GdkPixbuf *pixbuf,
 			    const gchar *name, const gchar *status,
-			    gpointer data)
+			    const gchar *progress_str, gpointer data)
 {
 	GtkListStore *store = progress->store;
 	GtkTreeIter iter;
@@ -186,13 +207,15 @@ void progress_dialog_append(ProgressDialog *progress, GdkPixbuf *pixbuf,
 			   PROG_COL_PIXBUF, pixbuf,
 			   PROG_COL_NAME, name,
 			   PROG_COL_STATUS, status,
+			   PROG_COL_PROGRESS, progress_str,
 			   PROG_COL_POINTER, data,
 			   -1);
 }
 
 void progress_dialog_set_row(ProgressDialog *progress, gint row,
 			     GdkPixbuf *pixbuf, const gchar *name,
-			     const gchar *status, gpointer data)
+			     const gchar *status, const gchar *progress_str,
+			     gpointer data)
 {
 	GtkListStore *store = progress->store;
 	GtkTreeIter iter;
@@ -203,6 +226,7 @@ void progress_dialog_set_row(ProgressDialog *progress, gint row,
 				   PROG_COL_PIXBUF, pixbuf,
 				   PROG_COL_NAME, name,
 				   PROG_COL_STATUS, status,
+				   PROG_COL_PROGRESS, progress_str,
 				   PROG_COL_POINTER, data,
 				   -1);
 	}
@@ -244,6 +268,19 @@ void progress_dialog_set_row_status(ProgressDialog *progress, gint row,
 	}
 }
 
+void progress_dialog_set_row_progress(ProgressDialog *progress, gint row,
+				      const gchar *progress_str)
+{
+	GtkListStore *store = progress->store;
+	GtkTreeIter iter;
+
+	if (gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(store),
+					  &iter, NULL, row)) {
+		gtk_list_store_set(store, &iter, PROG_COL_PROGRESS,
+				   progress_str, -1);
+	}
+}
+
 void progress_dialog_scroll_to_row(ProgressDialog *progress, gint row)
 {
 	GtkTreeModel *model = GTK_TREE_MODEL(progress->store);
